package com.ycwl.basic.service.impl.mobile;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ycwl.basic.constant.BaseContextHandler;
import com.ycwl.basic.constant.NumberConstant;
import com.ycwl.basic.mapper.pc.*;
import com.ycwl.basic.model.jwt.JwtInfo;
import com.ycwl.basic.model.mobile.goods.GoodsPageVO;
import com.ycwl.basic.model.mobile.index.TopStateResp;
import com.ycwl.basic.model.mobile.scenic.ScenicAppVO;
import com.ycwl.basic.model.mobile.scenic.ScenicDeviceCountVO;
import com.ycwl.basic.model.mobile.scenic.ScenicIndexVO;
import com.ycwl.basic.model.mobile.scenic.content.ContentPageVO;
import com.ycwl.basic.model.pc.face.req.FaceReqQuery;
import com.ycwl.basic.model.pc.face.resp.FaceRespVO;
import com.ycwl.basic.model.pc.scenic.req.ScenicReqQuery;
import com.ycwl.basic.model.pc.scenic.resp.ScenicRespVO;
import com.ycwl.basic.model.pc.source.req.SourceReqQuery;
import com.ycwl.basic.model.pc.source.resp.SourceRespVO;
import com.ycwl.basic.model.pc.template.req.TemplateReqQuery;
import com.ycwl.basic.model.pc.template.resp.TemplateRespVO;
import com.ycwl.basic.model.pc.video.req.VideoReqQuery;
import com.ycwl.basic.model.pc.video.resp.VideoRespVO;
import com.ycwl.basic.service.mobile.AppScenicService;
import com.ycwl.basic.utils.ApiResponse;
import com.ycwl.basic.utils.JwtTokenUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * @Author：longbinbin
 * @Date：2024/12/6 10:23
 */
@Slf4j
@Service
public class AppScenicServiceImpl implements AppScenicService {

    @Autowired
    private ScenicMapper scenicMapper;
    @Autowired
    private DeviceMapper deviceMapper;
    @Autowired
    private FaceMapper faceMapper;
    @Autowired
    private SourceMapper sourceMapper;
    @Autowired
    private TaskMapper taskMapper;
    @Autowired
    private VideoMapper videoMapper;
    @Autowired
    private TemplateMapper templateMapper;

    @Value("${face.score}")
    private BigDecimal faceScore;

    @Override
    public ApiResponse<PageInfo<ScenicAppVO>> pageQuery(ScenicReqQuery scenicReqQuery) {
        PageHelper.startPage(scenicReqQuery.getPageNum(), scenicReqQuery.getPageSize());
        List<ScenicAppVO> list = scenicMapper.appList(scenicReqQuery);
        PageInfo<ScenicAppVO> pageInfo = new PageInfo<>(list);
        return ApiResponse.success(pageInfo);
    }

    @Override
    public ApiResponse<ScenicDeviceCountVO> deviceCountByScenicId(Long scenicId) {
        JwtInfo worker = JwtTokenUtil.getWorker();
        ScenicDeviceCountVO scenicDeviceCountVO = deviceMapper.deviceCountByScenicId(scenicId, worker.getUserId());
        return ApiResponse.success(scenicDeviceCountVO);
    }

    @Override
    public ApiResponse<ScenicRespVO> getDetails(Long id) {
        ScenicRespVO scenicRespVO = scenicMapper.getAppById(id);
        ScenicDeviceCountVO scenicDeviceCountVO = deviceMapper.deviceCountByScenicId(id, -1L);
        scenicRespVO.setLensNum(scenicDeviceCountVO.getTotalDeviceCount());
        return ApiResponse.success(scenicRespVO);
    }

    @Override
    public ApiResponse<List<ScenicAppVO>> scenicList(ScenicIndexVO scenicIndexVO) {
        List<ScenicAppVO> dataList = scenicMapper.scenicListByLnLa(scenicIndexVO);
        return ApiResponse.success(dataList);
    }

    @Override
    public ApiResponse<TopStateResp> topState() {
        TopStateResp topStateResp = new TopStateResp();
        String userId = BaseContextHandler.getUserId();
        if (StringUtils.isBlank(userId)) {
            return ApiResponse.success(topStateResp);
        }
        // 查询是否有人脸
        FaceReqQuery faceReqQuery = new FaceReqQuery();
        faceReqQuery.setMemberId(Long.parseLong(userId));
        List<FaceRespVO> list = faceMapper.list(faceReqQuery);
        for (FaceRespVO faceRespVO : list) {
            if (faceRespVO.getScore().compareTo(faceScore) >= NumberConstant.ZERO) {
                topStateResp.setIsFace(true);
            }
        }

        // 查询是否有专属视频待制作
        int sourceNum = sourceMapper.countByMemberId(userId);
        topStateResp.setMadeVideos(sourceNum > 0);

        // 是否有专属视频合成中
        int taskSynthesizingNum = taskMapper.countByMemberIdStau(userId);
        topStateResp.setCompositingVideo(taskSynthesizingNum > 0);

        // AI合成了多少个视频 完成
        int taskFinishNum = taskMapper.countByMemberIdStauFinish(userId);
        topStateResp.setAiVideoNum(taskFinishNum);

        return ApiResponse.success(topStateResp);
    }

    @Override
    public ApiResponse<List<ContentPageVO>> contentList(Long scenicId) {
        JwtInfo worker = JwtTokenUtil.getWorker();
        //查询原素材
        List<ContentPageVO> contentList = new ArrayList<>();

        HashMap<Long, String> videoMap = new HashMap<>();
        VideoReqQuery videoReqQuery = new VideoReqQuery();
        videoReqQuery.setScenicId(scenicId);
        videoReqQuery.setMemberId(worker.getUserId());
        //查询成片vlog
        List<VideoRespVO> videoList = videoMapper.list(videoReqQuery);
        videoList.forEach(videoRespVO -> {
            ContentPageVO contentPageVO = new ContentPageVO();
            contentPageVO.setName(videoRespVO.getTemplateName());
            contentPageVO.setContentType(1);
            contentPageVO.setContentId(videoRespVO.getId());
            contentPageVO.setTemplateCoverUrl(videoRespVO.getTemplateCoverUrl());
            contentPageVO.setIsBuy(videoRespVO.getIsBuy());
            contentList.add(contentPageVO);
            videoMap.put(videoRespVO.getTemplateId(),"");
        });

        TemplateReqQuery templateReqQuery = new TemplateReqQuery();
        templateReqQuery.setScenicId(scenicId);
        List<TemplateRespVO> list = templateMapper.list(templateReqQuery);
        list.forEach(templateRespVO -> {
            Long templateId = templateRespVO.getId();
            if (!videoMap.containsKey(templateId)) {
                ContentPageVO contentPageVO = new ContentPageVO();
                contentPageVO.setName(templateRespVO.getName());
                contentPageVO.setContentType(0);
                contentPageVO.setContentId(templateId);
                contentPageVO.setTemplateCoverUrl(templateRespVO.getCoverUrl());
                contentList.add(contentPageVO);
            }
        });

        SourceReqQuery sourceReqQuery = new SourceReqQuery();
        videoReqQuery.setScenicId(scenicId);
        videoReqQuery.setMemberId(worker.getUserId());
        //查询源素材
        List<SourceRespVO> sourceList = sourceMapper.listGroupByType(sourceReqQuery);
        sourceList.forEach(sourceRespVO -> {
            ContentPageVO contentPageVO = new ContentPageVO();
            Integer type = sourceRespVO.getType();
            if(type==1){
                contentPageVO.setName("原片集");
            }else {
                contentPageVO.setName("照片集");
            }
            contentPageVO.setContentType(2);
            contentPageVO.setSourceType(type);
            contentPageVO.setIsBuy(sourceRespVO.getIsBuy());
            contentList.add(contentPageVO);
        });

        return ApiResponse.success(contentList);
    }
}
