package com.ycwl.basic.service.impl.mobile;

import com.ycwl.basic.mapper.pc.*;
import com.ycwl.basic.model.jwt.JwtInfo;
import com.ycwl.basic.model.mobile.goods.*;
import com.ycwl.basic.model.mobile.scenic.ScenicDeviceCountVO;
import com.ycwl.basic.model.pc.scenic.resp.ScenicRespVO;
import com.ycwl.basic.model.pc.source.req.SourceReqQuery;
import com.ycwl.basic.model.pc.source.resp.SourceRespVO;
import com.ycwl.basic.model.pc.task.req.TaskReqQuery;
import com.ycwl.basic.model.pc.video.req.VideoReqQuery;
import com.ycwl.basic.model.pc.video.resp.VideoRespVO;
import com.ycwl.basic.service.mobile.GoodsService;
import com.ycwl.basic.utils.ApiResponse;
import com.ycwl.basic.utils.DateUtils;
import com.ycwl.basic.utils.JwtTokenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author：longbinbin
 * @Date：2024/12/5 15:04
 */@Service
public class GoodsServiceImpl implements GoodsService {
     @Autowired
     private VideoMapper videoMapper;
     @Autowired
     private SourceMapper sourceMapper;
     @Autowired
     private ScenicMapper scenicMapper;
     @Autowired
     private TaskMapper taskMapper;
     @Autowired
     private DeviceMapper deviceMapper;

     public ApiResponse<List<GoodsPageVO>> goodsList(GoodsReqQuery query) {
         JwtInfo worker = JwtTokenUtil.getWorker();
         //查询原素材
         List<GoodsPageVO> goodsList = new ArrayList<>();

         VideoReqQuery videoReqQuery = new VideoReqQuery();
         videoReqQuery.setScenicId(query.getScenicId());
         videoReqQuery.setIsBuy(query.getIsBuy());
         videoReqQuery.setMemberId(worker.getUserId());
         //查询成片vlog
         List<VideoRespVO> videoList = videoMapper.list(videoReqQuery);
         videoList.forEach(videoRespVO -> {
             GoodsPageVO goodsPageVO = new GoodsPageVO();
             goodsPageVO.setGoodsName(videoRespVO.getTemplateName());
             goodsPageVO.setScenicId(videoRespVO.getScenicId());
             goodsPageVO.setScenicName(videoRespVO.getScenicName());
             goodsPageVO.setLongitude(videoRespVO.getLongitude());
             goodsPageVO.setLatitude(videoRespVO.getLatitude());
             goodsPageVO.setGoodsType(1);
             goodsPageVO.setGoodsId(videoRespVO.getId());
             goodsPageVO.setTemplateCoverUrl(videoRespVO.getTemplateCoverUrl());
             goodsList.add(goodsPageVO);
         });


         SourceReqQuery sourceReqQuery = new SourceReqQuery();
         sourceReqQuery.setScenicId(query.getScenicId());
         sourceReqQuery.setIsBuy(query.getIsBuy());
         sourceReqQuery.setMemberId(query.getMemberId());
         //查询源素材
         List<SourceRespVO> sourceList = sourceMapper.listGroupByType(sourceReqQuery);
         sourceList.forEach(sourceRespVO -> {
             GoodsPageVO goodsPageVO = new GoodsPageVO();
             Integer type = sourceRespVO.getType();
             if(type==1){
                 goodsPageVO.setGoodsName("原片集");
             }else {
                 goodsPageVO.setGoodsName("照片集");
             }
             goodsPageVO.setScenicId(sourceRespVO.getScenicId());
             goodsPageVO.setScenicName(sourceRespVO.getScenicName());
             goodsPageVO.setLongitude(sourceRespVO.getLongitude());
             goodsPageVO.setLatitude(sourceRespVO.getLatitude());
             goodsPageVO.setGoodsType(2);
             goodsPageVO.setSourceType(type);
             goodsList.add(goodsPageVO);
         });

         return ApiResponse.success(goodsList);
     }

    @Override
    public ApiResponse<List<GoodsDetailVO>> sourceGoodsList(GoodsReqQuery query) {
        Integer sourceType = query.getSourceType();
        SourceReqQuery sourceReqQuery = new SourceReqQuery();
         sourceReqQuery.setScenicId(query.getScenicId());
         sourceReqQuery.setIsBuy(query.getIsBuy());
         sourceReqQuery.setMemberId(query.getMemberId());
         sourceReqQuery.setType(sourceType);
        List<SourceRespVO> list = sourceMapper.list(sourceReqQuery);
        List<GoodsDetailVO> goodsDetailVOList = new ArrayList<>();

        String goodsNamePrefix = "";
        if(sourceType==1){
            goodsNamePrefix="视频";
        }else if (sourceType==2){
            goodsNamePrefix="图片";
        }else {
            goodsNamePrefix="其他类型";
        }
        //图片编号
        int i=1;
        for (SourceRespVO sourceRespVO : list) {
            GoodsDetailVO goodsDetailVO = new GoodsDetailVO();
            goodsDetailVO.setGoodsId(sourceRespVO.getId());
            String shootingTime = DateUtils.format(sourceRespVO.getCreateTime(), "yyyy.MM.dd HH:mm:ss");
            if(i<10){
                goodsDetailVO.setGoodsName(goodsNamePrefix +"0"+i+" "+shootingTime);
            }else {
                goodsDetailVO.setGoodsName(goodsNamePrefix +i+" "+shootingTime);
            }
            goodsDetailVO.setScenicId(sourceRespVO.getScenicId());
            goodsDetailVO.setScenicName(sourceRespVO.getScenicName());
            goodsDetailVO.setLongitude(sourceRespVO.getLongitude());
            goodsDetailVO.setLatitude(sourceRespVO.getLatitude());
            goodsDetailVO.setGoodsType(2);
            goodsDetailVO.setSourceType(sourceType);
            goodsDetailVO.setGoodsId(sourceRespVO.getId());
            goodsDetailVO.setVideoUrl(sourceRespVO.getVideoUrl());
            goodsDetailVO.setUrl(sourceRespVO.getUrl());
            goodsDetailVO.setCreateTime(sourceRespVO.getCreateTime());

            goodsDetailVOList.add(goodsDetailVO);
            i++;
        }

        return ApiResponse.success(goodsDetailVOList);
    }

    @Override
    public ApiResponse<BigDecimal> queryPrice(GoodsPriceQueryReq queryPriceData) {
        Integer goodsType = queryPriceData.getGoodsType();

        if(goodsType==1){//成片vlog
            //成片的价格就是成片所用template的价格
            Long videoId = queryPriceData.getGoodsId();
            VideoRespVO videoRespVO = videoMapper.getById(videoId);
            if(videoRespVO==null){
                return ApiResponse.fail("该vlog不存在或已失效");
            }
            BigDecimal tmplatePrice = videoRespVO.getTemplatePrice();
            if(tmplatePrice==null){
                return ApiResponse.fail("该vlog使用的模板价格或状态异常，请联系管理员");
            }
            return ApiResponse.success(tmplatePrice);

        }else if(goodsType==2){//原素材
            //原素材的价格就是原素材对应景区定的价格
            Long scenicId = queryPriceData.getScenicId();
            ScenicRespVO scenicRespVO = scenicMapper.getById(scenicId);
            if(scenicRespVO==null){
                return ApiResponse.fail("该景区不存在或状态异常，请联系管理员");
            }
            BigDecimal price = scenicRespVO.getPrice();
            if(price==null){
                return ApiResponse.fail("该景区的原片价格未设定或状态异常，请联系管理员");
            }
            return ApiResponse.success(price);
        }
        return ApiResponse.fail("不合法的商品，请联系管理员");
    }

    @Override
    public ApiResponse<VideoGoodsDetailVO> videoGoodsDetail(Long goodsId) {
        VideoGoodsDetailVO goodsDetailVO = new VideoGoodsDetailVO();
        VideoRespVO videoRespVO = videoMapper.getById(goodsId);
        if(videoRespVO==null){
            return ApiResponse.fail("该vlog不存在或已失效");
        }
        goodsDetailVO.setGoodsName(videoRespVO.getTemplateName());
        goodsDetailVO.setScenicId(videoRespVO.getScenicId());
        goodsDetailVO.setScenicName(videoRespVO.getScenicName());
        goodsDetailVO.setLongitude(videoRespVO.getLongitude());
        goodsDetailVO.setLatitude(videoRespVO.getLatitude());
        goodsDetailVO.setGoodsType(1);
        goodsDetailVO.setGoodsId(videoRespVO.getId());
        goodsDetailVO.setVideoUrl(videoRespVO.getVideoUrl());
        goodsDetailVO.setTemplateCoverUrl(videoRespVO.getTemplateCoverUrl());
        goodsDetailVO.setCreateTime(videoRespVO.getCreateTime());
        BigDecimal templatePrice = videoRespVO.getTemplatePrice();
        // 使用DecimalFormat格式化输出
        DecimalFormat df = new DecimalFormat("0.00");
        goodsDetailVO.setPrice(templatePrice==null?"":df.format(templatePrice.setScale(2, RoundingMode.HALF_UP)));
        goodsDetailVO.setIsBuy(videoRespVO.getIsBuy());
        ScenicDeviceCountVO scenicDeviceCountVO = deviceMapper.deviceCountByScenicId(videoRespVO.getScenicId(), -1L);
        goodsDetailVO.setLensNum(scenicDeviceCountVO.getTotalDeviceCount());
        return ApiResponse.success(goodsDetailVO);
    }

    /**
     * 查询用户当前景区的视频合成任务状态
     * @param scenicId 景区id
     * @return 0没有任务 1 合成中 2 合成成功
     */
    @Override
    public ApiResponse<Integer> getAllTaskStatus(Long userId, Long scenicId) {
        TaskReqQuery taskReqQuery = new TaskReqQuery();
        taskReqQuery.setMemberId(userId);
        taskReqQuery.setScenicId(scenicId);
        int allCount=taskMapper.countTask(taskReqQuery);
        if(allCount==0){//没有任务
            return ApiResponse.success(0);
        }
        taskReqQuery.setStatus(2);
        int count=taskMapper.countTask(taskReqQuery);

        if(count>0){//有任务在进行中
            return ApiResponse.success(1);
        }else {//没有任务在进行中
            return ApiResponse.success(2);
        }
    }

    @Override
    public ApiResponse<Integer> getTemplateTaskStatus(Long userId, Long templateId) {
        TaskReqQuery taskReqQuery = new TaskReqQuery();
        taskReqQuery.setMemberId(userId);
        taskReqQuery.setTemplateId(templateId);
        int allCount=taskMapper.countTask(taskReqQuery);
        if(allCount==0){//没有任务
            return ApiResponse.success(0);
        }
        taskReqQuery.setStatus(2);
        int count=taskMapper.countTask(taskReqQuery);
        if(count>0){
            return ApiResponse.success(1);
        }else {
            return ApiResponse.success(2);
        }
    }

}
