package com.ycwl.basic.service.impl.mobile;


import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.utils.StringUtils;
import com.wechat.pay.java.core.Config;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.core.exception.ServiceException;
import com.wechat.pay.java.core.http.HttpMethod;
import com.wechat.pay.java.core.notification.NotificationConfig;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.core.util.PemUtil;
import com.wechat.pay.java.service.partnerpayments.jsapi.model.Transaction;
import com.wechat.pay.java.service.payments.jsapi.JsapiService;
import com.wechat.pay.java.service.payments.jsapi.model.*;
import com.wechat.pay.java.service.payments.model.TransactionAmount;
import com.ycwl.basic.config.WechatConfig;
import com.ycwl.basic.constant.HttpConstant;
import com.ycwl.basic.constant.NumberConstant;
import com.ycwl.basic.constant.WeiXinConstant;
import com.ycwl.basic.enums.BizCodeEnum;
import com.ycwl.basic.enums.OrderStateEnum;
import com.ycwl.basic.exception.AppException;
import com.ycwl.basic.model.pc.order.resp.OrderRespVO;
import com.ycwl.basic.model.wx.WXPayOrderReqVO;
import com.ycwl.basic.model.wx.WxPayRespVO;
import com.ycwl.basic.model.wx.WxchatCallbackSuccessData;
import com.ycwl.basic.service.HttpService;
import com.ycwl.basic.service.mobile.WxPayService;
import com.ycwl.basic.service.pc.OrderService;
import com.ycwl.basic.utils.SnowFlakeUtil;
import com.ycwl.basic.utils.WXPayUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.crypto.Cipher;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.GCMParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.security.*;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static com.wechat.pay.java.core.http.Constant.*;
import static com.ycwl.basic.constant.WeiXinConstant.*;

/**
 * @Author: songmingsong
 * @CreateTime: 2024-12-05
 * @Description: 微信支付实现
 * @Version: 1.0
 */
@Slf4j
@Service
public class WxPayServiceImpl implements WxPayService {

    @Autowired
    private WechatConfig wechatConfig;
    @Autowired
    private HttpService httpService;
    @Autowired
    private OrderService orderService;

    @Override
    public WxPayRespVO createOrder(WXPayOrderReqVO req) {
        try {
            // 使用自动更新平台证书的RSA配置
            Config config = getInstance(wechatConfig);
            // 构建service
            JsapiService service = new JsapiService.Builder().config(config).build();

            // request.setXxx(val)设置所需参数，具体参数可见Request定义
            PrepayRequest request = new PrepayRequest();
            Amount amount = new Amount();
            amount.setTotal(req.getTotalPrice());
            request.setAmount(amount);
            request.setAppid(wechatConfig.getAppId());
            request.setMchid(wechatConfig.getMchId());
            request.setDescription(req.getGoodsName());
            request.setNotifyUrl(wechatConfig.getPayNotifyUrl());
            request.setOutTradeNo(req.getOrderSn().toString());

            Payer payer = new Payer();
            payer.setOpenid(req.getOpenId());
            request.setPayer(payer);

            // 调用下单方法，得到应答
            PrepayResponse response = service.prepay(request);
            WxPayRespVO vo = new WxPayRespVO();
            Long timeStamp = System.currentTimeMillis() / NumberConstant.THOUSAND;
            vo.setTimeStamp(timeStamp);
            String substring = UUID.randomUUID().toString().replaceAll("-", "").substring(0, 32);
            vo.setNonceStr(substring);
            String signatureStr = Stream.of(wechatConfig.getAppId(), String.valueOf(timeStamp), substring, "prepay_id=" + response.getPrepayId())
                    .collect(Collectors.joining("\n", "", "\n"));
            String sign = WXPayUtil.getSign(signatureStr, wechatConfig.getKeyPath());
            vo.setPaySign(sign);
            vo.setSignType("RSA-SHA256");
            vo.setPrepayId("prepay_id=" + response.getPrepayId());
            return vo;
        } catch (ServiceException e) {
            JSONObject parse = JSONObject.parseObject(e.getResponseBody());
            throw new AppException(BizCodeEnum.ADVANCE_PAYMENT_FAILED, parse.getString(HttpConstant.Message));
        } catch (Exception e) {
            throw new AppException(BizCodeEnum.ADVANCE_PAYMENT_FAILED, e.toString());
        }
    }

    @Override
    public void payNotify(HttpServletRequest request) {
        try {
            // 读取请求体的信息
            ServletInputStream inputStream = request.getInputStream();
            StringBuffer stringBuffer = new StringBuffer();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
            String s;
            // 读取回调请求体
            while ((s = bufferedReader.readLine()) != null) {
                stringBuffer.append(s);
            }
            String s1 = stringBuffer.toString();
            String timestamp = request.getHeader(WECHAT_PAY_TIMESTAMP);
            String nonce = request.getHeader(WECHAT_PAY_NONCE);
            String signType = request.getHeader(WeiXinConstant.WECHATPAY_SIGNATURE_TYPE);
            String serialNo = request.getHeader(WECHAT_PAY_SERIAL);
            String signature = request.getHeader(WECHAT_PAY_SIGNATURE);

            NotificationConfig config = new RSAAutoCertificateConfig.Builder()
                    .merchantId(wechatConfig.getMchId())
                    .privateKeyFromPath(wechatConfig.getKeyPath())
                    .merchantSerialNumber(wechatConfig.getMchSerialNo())
                    .apiV3Key(wechatConfig.getApiV3())
                    .build();
            // 初始化 NotificationParser
            NotificationParser parser = new NotificationParser(config);
            RequestParam requestParam = new RequestParam.Builder()
                    .serialNumber(serialNo)
                    .nonce(nonce)
                    .signature(signature)
                    .timestamp(timestamp)
                    // 若未设置signType，默认值为 WECHATPAY2-SHA256-RSA2048
                    .signType(signType)
                    .body(s1)
                    .build();
            Transaction parse = parser.parse(requestParam, Transaction.class);
            log.info("[微信支付]parse = {}", parse);

            // 更新订单信息
            new Thread(() -> {
                Transaction.TradeStateEnum tradeState = parse.getTradeState();
                OrderStateEnum OrderStateEnum = null;
                if (Transaction.TradeStateEnum.SUCCESS == tradeState) {
                    OrderStateEnum = OrderStateEnum.PAID;
                }
                if (Objects.nonNull(OrderStateEnum)) {
                    orderService.updateOrderState(Long.parseLong(parse.getOutTradeNo()), OrderStateEnum, null);
                }
            }).start();
        } catch (Exception e) {
            throw new AppException(BizCodeEnum.ADVANCE_PAYMENT_CALLBACK_FAILED, e.toString());
        }
    }

    @Override
    public WxchatCallbackSuccessData queryPay(Long orderId) {
        WxchatCallbackSuccessData wxchatCallbackSuccessData = new WxchatCallbackSuccessData();
        QueryOrderByOutTradeNoRequest queryRequest = new QueryOrderByOutTradeNoRequest();
        queryRequest.setMchid(wechatConfig.getMchId());
        queryRequest.setOutTradeNo(orderId.toString());
        Config config = getInstance(wechatConfig);
        // 构建service
        JsapiService service = new JsapiService.Builder().config(config).build();
        try {
            com.wechat.pay.java.service.payments.model.Transaction transaction = service.queryOrderByOutTradeNo(queryRequest);
            String successTime = transaction.getSuccessTime();

            wxchatCallbackSuccessData.setOrderId(orderId.toString());
            wxchatCallbackSuccessData.setSuccessTime(successTime);
            wxchatCallbackSuccessData.setTradetype(WeiXinConstant.getDescriptionType(transaction.getTradeType()));
            wxchatCallbackSuccessData.setTradestate(WeiXinConstant.getDescriptionState(transaction.getTradeState()));
            TransactionAmount amount = transaction.getAmount();
            Integer total = amount.getTotal();
            wxchatCallbackSuccessData.setTotalMoney(new BigDecimal(total).movePointLeft(2));
        } catch (ServiceException e) {
            // API返回失败, 例如ORDER_NOT_EXISTS
            log.error("[微信退款] code={}, message={}", e.getErrorCode(), e.getErrorMessage());
            log.error("[微信退款] Response_body={}", e.getResponseBody());
        }
        return wxchatCallbackSuccessData;
    }

    @Override
    public Boolean refundOrder(String orderId) throws Exception {
        OrderRespVO orderDetail = orderService.detail(Long.parseLong(orderId)).getData();
        BigDecimal payPrice = orderDetail.getPayPrice();
        int priceInCents = payPrice.multiply(new BigDecimal(NumberConstant.HUNDRED)).intValue(); // 转换为分（int）
        // 构建退款请求参数
        Map<String, Object> data = new HashMap<>();
        data.put(WECHATPAY_OUT_TRADE_NO, orderId);// 你要退款的订单号
        data.put(WECHATPAY_OUT_REFUND_NO, SnowFlakeUtil.getId()); // 退款订单号（随机生成一个就行，保证不重复）

        Map<String, Object> amount = new HashMap<>();
        amount.put(WECHATPAY_REFUND, priceInCents); // 退款金额（以分为单位）
        amount.put(WECHATPAY_TOTAL, priceInCents); // 订单总金额（以分为单位）
        amount.put(WECHATPAY_CURRENCY, WECHATPAY_CURRENCY_CNY);
        data.put(WECHATPAY_AMOUNT, amount);

        // 回调通知 URL
        data.put(WECHATPAY_NOTIFY_URL, wechatConfig.getRefundNotifyUrl());
        Map<String, Object> stringObjectMap = this.processRefund(data);
        if (stringObjectMap.get(WECHATPAY_STATUS).equals(WECHATPAY_SUCCESS)
                || stringObjectMap.get(WECHATPAY_STATUS).equals(WECHATPAY_PROCESSING)) {
            orderService.updateOrderState(Long.parseLong(orderId), OrderStateEnum.REFUNDED, null);
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean refundNotify(String refundResult) throws IOException, GeneralSecurityException {
        // 转为map格式
        Map<String, String> jsonMap = JSONObject.parseObject(refundResult, Map.class);

        /*
         * 退款成功后返回一个加密字段resource,以下为解密
         *  解密需要从resource参数中，获取到ciphertext，nonce，associated_data这三个参数进行解密
         */
        String resource = JSONObject.toJSONString(jsonMap.get(REFUNDS_RESOURCE));
        JSONObject object = JSONObject.parseObject(resource);

        String ciphertext = String.valueOf(object.get(REFUNDS_CIPHERTEXT));
        String nonce = String.valueOf(object.get(REFUNDS_NONCE));
        String associated_data = String.valueOf(object.get(REFUNDS_ASSOCIATED_DATA));

        String resultStr = decryptToString(associated_data.getBytes(String.valueOf(StandardCharsets.UTF_8)),
                nonce.getBytes(String.valueOf(StandardCharsets.UTF_8)),
                ciphertext);
        Map<String, String> reqInfo = JSONObject.parseObject(resultStr, Map.class);

        String refund_status = reqInfo.get(REFUNDS_REFUND_STATUS);// 退款状态
        String out_trade_no = reqInfo.get(WECHATPAY_OUT_TRADE_NO); // 订单号

        if (!StringUtils.isEmpty(refund_status) && WECHATPAY_SUCCESS.equals(refund_status)) {
            orderService.updateOrderState(Long.parseLong(out_trade_no), OrderStateEnum.REFUNDED, null);
            log.info("[微信退款回调]退款成功");
            return true;
        } else {
            log.error("[微信退款回调]退款失败");
            return false;
        }
    }

    @Override
    public void closeOrder(String orderId) {
        CloseOrderRequest closeOrderRequest = new CloseOrderRequest();
        closeOrderRequest.setOutTradeNo(orderId);
        closeOrderRequest.setMchid(wechatConfig.getMchId());
        Config config = getInstance(wechatConfig);
        // 构建service
        JsapiService service = new JsapiService.Builder().config(config).build();
        service.closeOrder(closeOrderRequest);
    }


    /**
     * 退款回调  解密数据
     *
     * @param associatedData
     * @param nonce
     * @param ciphertext
     * @return
     * @throws GeneralSecurityException
     * @throws IOException
     */
    public String decryptToString(byte[] associatedData, byte[] nonce, String ciphertext) throws GeneralSecurityException, IOException {
        try {
            Cipher cipher = Cipher.getInstance(HttpConstant.AES_GCM_NoPadding);

            SecretKeySpec key = new SecretKeySpec(wechatConfig.getKeyPath().getBytes(), HttpConstant.AES);
            GCMParameterSpec spec = new GCMParameterSpec(NumberConstant.ONE_HUNDRED_TWENTY_EIGHT, nonce);// 规定为128

            cipher.init(Cipher.DECRYPT_MODE, key, spec);
            cipher.updateAAD(associatedData);

            return new String(cipher.doFinal(Base64.getDecoder().decode(ciphertext)), String.valueOf(StandardCharsets.UTF_8));
        } catch (NoSuchAlgorithmException | NoSuchPaddingException e) {
            throw new IllegalStateException(e);
        } catch (InvalidKeyException | InvalidAlgorithmParameterException e) {
            throw new IllegalArgumentException(e);
        }
    }

    /**
     * 发起退款
     *
     * @param data 退款请求参数
     * @return
     * @throws Exception
     */
    public Map<String, Object> processRefund(Map<String, Object> data) throws Exception {
        // 请求头（需要根据业务生成 Authorization Token）
        Map<String, String> headers = new HashMap<>();
        headers.put(HttpConstant.Authorization, getToken(String.valueOf(HttpMethod.POST), REFUNDS_URL, JSONObject.toJSONString(data)));
        headers.put(HttpConstant.Accept, HttpConstant.Application_Json);
        headers.put(HttpConstant.Content_Type, HttpConstant.Application_Json);

        // 请求体数据
        String jsonData = JSONObject.toJSONString(data);

        // 执行 POST 请求
        String response = httpService.doPost(REFUNDS_URL, jsonData, headers, String.valueOf(StandardCharsets.UTF_8));

        // 将响应字符串解析为 Map
        Map<String, Object> responseMap = JSONObject.parseObject(response, Map.class);
        return responseMap;
    }

    /**
     * 生成退款请求token
     *
     * @param method
     * @param orderId
     * @param body
     * @return
     * @throws InvalidKeyException
     * @throws NoSuchAlgorithmException
     * @throws UnsupportedEncodingException
     * @throws SignatureException
     */
    public String getToken(String method, String orderId, String body) throws InvalidKeyException, NoSuchAlgorithmException, UnsupportedEncodingException, SignatureException {
        String nonceStr = WXPayUtil.generateNonceStr();
        long timestamp = System.currentTimeMillis() / NumberConstant.THOUSAND; // 生成时间戳

        String parameter = method + "\n"
                + REFUNDS_URi + orderId + "\n"
                + timestamp + "\n"
                + nonceStr + "\n"
                + body + "\n";

        // 对参数进行加密
        byte[] bytes = parameter.getBytes(String.valueOf(StandardCharsets.UTF_8));
        Signature sign = Signature.getInstance("SHA256withRSA");
        PrivateKey privateKey = PemUtil.loadPrivateKeyFromPath(wechatConfig.getKeyPath());  // privateKeyPath是商户证书密钥的位置apiclient_key.pem
        sign.initSign(privateKey);   // 商户密钥文件路径
        sign.update(bytes);
        String signature = Base64.getEncoder().encodeToString(sign.sign());

        // 获取token
        String token = "mchid=\"" + wechatConfig.getMchId() + "\","      // 商户号
                + "nonce_str=\"" + nonceStr + "\","
                + "timestamp=\"" + timestamp + "\","
                + "serial_no=\"" + wechatConfig.getMchSerialNo() + "\"," // merchantSerialNumber是微信支付中申请的证书序列号
                + "signature=\"" + signature + "\"";

        return REFUNDS_SCHEMA + token;
    }

    /**
     * 一个商户号只能初始化一个配置，否则会因为重复的下载任务报错
     */
    private static class Holder {
        private static volatile Config instance;

        static void init(WechatConfig wechatConfig) {
            instance = new RSAAutoCertificateConfig.Builder()
                    .merchantId(wechatConfig.getMchId())
                    .privateKeyFromPath(wechatConfig.getKeyPath())
                    .merchantSerialNumber(wechatConfig.getMchSerialNo())
                    .apiV3Key(wechatConfig.getApiV3())
                    .build();
        }
    }

    public static Config getInstance(WechatConfig wechatConfig) {
        if (Holder.instance == null) {
            synchronized (Holder.class) {
                if (Holder.instance == null) {
                    Holder.init(wechatConfig);
                }
            }
        }
        return Holder.instance;
    }
}
